package org.jvnet.hifaces.text;

import java.text.MessageFormat;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.faces.context.FacesContext;
import javax.faces.el.EvaluationException;
import javax.faces.el.PropertyNotFoundException;
import javax.faces.el.ValueBinding;

public abstract class AbstractText extends ValueBinding {

	@Override
	public Class getType(FacesContext facesContext) throws EvaluationException,
			PropertyNotFoundException {
		return String.class;
	}

	/**
	 * Return the textual value. The returned value is a string. This
	 * implementation retrieves the resource bundle and passes it to the
	 * <code>getText(...)</code> method. Thrown exceptions are rethrown ass
	 * evaluation exceptions.
	 */
	@Override
	public Object getValue(FacesContext facesContext)
			throws EvaluationException, PropertyNotFoundException {
		try {
			final ResourceBundle resourceBundle = getResourceBundle(facesContext);
			final String text = getText(facesContext, resourceBundle);
			return text;

		} catch (Exception ex) {
			throw new EvaluationException(ex);
		}
	}

	private transient ResourceBundle resourceBundle;

	/**
	 * Returns the resource bundle (already loaded or newly retrieved).
	 * 
	 * @param facesContext
	 *            Faces context.
	 * @return Resource bundle.
	 * @throws Exception
	 *             Implementations may throw generic exceptions.
	 */
	protected ResourceBundle getResourceBundle(FacesContext facesContext)
			throws Exception {
		if (resourceBundle == null) {
			resourceBundle = createResourceBundle(facesContext);
		}
		return resourceBundle;
	}

	/**
	 * Retrieves a resource bundle for the given context. This implementation
	 * returns the application bundle. Subclasses may provide alternative
	 * implementations.
	 * 
	 * @param facesContext
	 *            Faces context.
	 * @return Resource bundle for the given context.
	 * @throws Exception
	 */
	protected ResourceBundle createResourceBundle(FacesContext facesContext)
			throws Exception {
		final String draftMessageBundle = facesContext.getApplication()
				.getMessageBundle();
		final String messageBundle = draftMessageBundle != null ? draftMessageBundle
				: getDefaultMessageBundle();
		if (messageBundle != null) {
			final ResourceBundle bundle = ResourceBundle
					.getBundle(messageBundle);
			return bundle;
		} else {
			return null;
		}
	}

	public String getDefaultMessageBundle() {
		return "messages";
	}

	/**
	 * An utility method to resolve the key against the resource bundle.
	 * 
	 * @param facesContext
	 *            Faces context.
	 * @param resourceBundle
	 *            resource bundle to use for resolution.
	 * @param messageId
	 *            id of the message to resolve.
	 * @param arguments
	 *            formatting arguments, may be <code>null</code>.
	 * @return Resolved textual content.
	 */
	protected String getText(final FacesContext facesContext,
			final ResourceBundle resourceBundle, final String messageId,
			final Object[] arguments) {
		try {
			final String message = resourceBundle.getString(messageId);

			if (arguments == null || arguments.length == 0) {
				return message;
			} else {
				try {
					return MessageFormat.format(message, arguments);
				} catch (Exception ex) {
					return "???" + messageId + "???";
				}
			}
		} catch (MissingResourceException ex) {
			return "???" + messageId + "???";
		}
	}

	/**
	 * Returns text content.
	 * 
	 * @param facesContext
	 *            Faces context.
	 * @param resourceBundle
	 *            resource bundle which may be used for text resolution.
	 * @return Textual content.
	 * @throws Exception
	 *             Implementations may throw generic exceptions.
	 */
	protected abstract String getText(FacesContext facesContext,
			ResourceBundle resourceBundle) throws Exception;

	@Override
	public final boolean isReadOnly(FacesContext facesContext)
			throws EvaluationException, PropertyNotFoundException {
		return true;
	}

	@Override
	public final void setValue(FacesContext facesContext, Object value)
			throws EvaluationException, PropertyNotFoundException {
	}

}
